/*
 * Copyright (c) 2017, Citrix Systems Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/*

EMP commands take the following format:

    {"execute" : "<command>", "arguments": { "arg1" : "val1:"} }

and you should expect:

    {"return" : {} }

or

    { "error" : "descriptive error" }

-------------------------------------------------------------------------
Command           Description
----------------  -------------------------------------------------------
migrate_init      An FD is sent via the auxiliary channel, which
                  shoud be used later for migration.

migrate_live      The emulator may start migrating, while the guest
                  is still running.  This call should come after a
                  migrate_init and track_dirty command.

migrate_nonlive   The emulator should start migrating, the guest
                  should not be running.  This must come after a migrate_init.

migrate_pause     Following on from a migrate_live, the VM has been paused,
                  and no further writes to its ram may be made.

migrate_paused    After all emus have been asked to 'pause', this command
                  this command indicates, the VM is paused, and you can
                  expect no furter writes to its ram from any emu.

migrate_progress  The emulator should provide updates on
                  migration progress.
                  Arguments may include the triggers.
                  “data-sent”: <minimum sent in bytes>

migrate_abort     Due to some sort of error, migration should stop,
                  and normal operation resume - if possible.

quit              The emulator should terminate.

restore           The emulator is to restore state, created using a
                  migrate command above.  This must follow a migrate_init.

set_args          Optional command, allows emu specific arguments
                  to be sent to an emu.

track_dirty       This instructs the emulator to start dirty page tracking.
-------------------------------------------------------------------------


After an migrate_progress, repeated responses will be received as follows.
Note that the iterations field is optional.  Also not that if the bytes
"remaining" in not known (perhaps becouse you are mid iteration), '-1' can be
sent.

{
   "event" : "MIGRATION", "data":
       {
       "sent" : 500,
       "remaning" : 300
       "iteration": 3
       }
}

After migation has completed, a repsonse like below may be recived. (Note that
a result is optional)

{
   "event" : "MIGRATION", "data":
       {
       "status": "completed",
       "result": "0 0"
       }
}

*/

#ifndef EMP_H
#define EMP_H

#include <stdbool.h>

/*
 * This is an index into the commands array defined in emp.c. It must be kept
 * in sync.
 */
enum command_num {
    cmd_migrate_abort,
    cmd_migrate_init,
    cmd_migrate_live,
    cmd_migrate_nonlive,
    cmd_migrate_pause,
    cmd_migrate_paused,
    cmd_migrate_progress,
    cmd_quit,
    cmd_restore,
    cmd_set_args,
    cmd_track_dirty,
    cmd_number
};

struct command {
    char *name;
    bool needs_fd;
};

const struct command *command_from_num(enum command_num cmd_num);
enum command_num command_num_from_string(const char *str);

#endif
