/*
 * Copyright (c) 2017, Citrix Systems Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#ifndef LIBEMP_H
#define LIBEMP_H

#include <sys/select.h>
#include <poll.h>
#include "emp.h"

enum emp_migration_status
{
    migration_success,
    migration_failed
};

struct emu_client { int num; };
struct json_object;

struct emp_call_args_s
{
    struct emu_client cli;
    int fd;
    struct json_object *cmd_args;
    void (*func)(struct emp_call_args_s *);
};


typedef struct emp_call_args_s emp_call_args;

struct command_actions
{
    enum command_num cmd;
    void (*func)(emp_call_args *);
    int in_thread;
};

struct emp_sock_inf;

/* Functions to be called from a main select/poll loop */

int emp_get_default_path(char *socket_path, size_t size, const char *emu_name, const int domid);
int emp_sock_init(const char *name, struct emp_sock_inf **inf, struct command_actions actions[]);
int emp_sock_close(struct emp_sock_inf **inf);

int emp_select_fdset(struct emp_sock_inf *inf, fd_set *fds, int *num_clients);
int emp_select_fdread(struct emp_sock_inf *inf, fd_set *fds, int activity);

int emp_poll_fdset(struct emp_sock_inf *inf, struct pollfd **fds, nfds_t *nfds, nfds_t *max_nfds, int *num_clients);
int emp_poll_fdread(struct emp_sock_inf *inf, struct pollfd *fds, nfds_t nfds, int activity);

/* Functions to be called from callbacks */

int emp_send_event_migrate_completed(struct emu_client cli, enum emp_migration_status status);
int emp_send_event_migrate_completed_result(struct emu_client cli, char *result);
int emp_send_event_migrate_progress(struct emu_client cli, int sent, int remaining, int iter);
int emp_send_event(struct emu_client cli, char *ret);
int emp_send_return(struct emu_client cli, char *ret);
int emp_send_error(struct emu_client cli, char *ret);

/* Logging */

enum emp_log_level
{
    emp_level_dbg,
    emp_level_info,
    emp_level_warn,
    emp_level_err,
};

typedef void (*emp_log_cb_t)(enum emp_log_level level, const char *msg);

void emp_set_log_cb(emp_log_cb_t cb);

/* Global emp lock */

int emp_lock(void);
int emp_unlock(void);

#endif /* LIBEMP_H */
